#
# wx_weekly.py
# For Python 3.6.2
#
# Purpose: Accesses the daily NOAA weather files for the past week
# and calculate the necessary averages, highs, lows, and provides north-wind
# and low-wind observations for the science techs weekly report.
#
# This version of the script looks in the directory:
#   \\Server\ftp\data\GEOSummit\NOAA\MetData\Sonic Data
# And identifies files with names structured:
#   NOAAWx_YYYY-MM-DD.dat
#
# These files are generated by the noaa-wx.py script, which saves observations from the Summit weather UDP stream.
#
# It will import files for the last seven days. Note that the day-of-week labels assume this script is being
# executed on a Monday (during typical reporting). If this is not the case, the day-of-week labels will be incorrect.
#
# Input: Data files must present in appropriate folder, as described above.
#
# Output:
#
#   weeklyWx_YYYY-MM-DD.txt is a text file organized into a table that shows the highs, lows,
#   and averages for each day in the past week, as well as north-wind and low-wind times.
#
# Example:
#
# From the command prompt, execute:
# python wx_weekly.py
#
# Initial script written by Aurora, ~2017-05-23.
# Modified by Sam Dorsi, 2017-08-08. Adapted for new NOAA met system, added supplemental commenting,
#   added error handling for missing files.
# Modified by Sam Dorsi, 2017-10-04. Added header content.
#
# Future projects:  check if New Year accounting is working
#                   use standard tools not BYO date formatting
#                   improve date handling (e.g., accurate DOW if not executed on a Monday)
#                   


import datetime
import time
from time import localtime
import calendar
import numpy as np

#Create function to convert Julian day to standard date. This function
#takes in the inputs of the year and the julian day and spits out the
#date in the format used by the wx files naming convention
def JulianDate_to_YYMMDD(y,jd):
    month = 1
    day = 0
    while jd - calendar.monthrange(y,month)[1] > 0 and month <= 12:
        jd = jd - calendar.monthrange(y,month)[1]
        month = month + 1
    if month > 9 and jd > 9:    
        return str(y)+'-'+str(month)+'-'+str(jd)
    if month < 10 and jd > 9:
        return str(y)+'-'+'0'+str(month)+'-'+str(jd)
    if month < 10 and jd < 10:
        return str(y)+'-'+'0'+str(month)+'-'+'0'+str(jd)
    if month > 9 and jd < 10:
        return str(y)+'-'+str(month)+'-'+'0'+ str(jd)
    
#Get current date and julian day and print
current_date = time.strftime("%Y-%m-%d")
current_julian_day = localtime().tm_yday
current_year = localtime().tm_year

print('Current date: ' + current_date)
print('Current day of year: ' + str(current_julian_day))

#Get start and end date for the wx files needed
#Get start date if week does not straddle the new years
if current_julian_day > 7:
    start_jd = current_julian_day - 7;
    start_date = JulianDate_to_YYMMDD(current_year,start_jd)

#This case allows for if a week of wx straddles the new year
if current_julian_day < 8:
    if calendar.isleap(current_year - 1):
        start_jd = 366 + (current_julian_day - 7) #if leap year, then julian day count is 366
        start_date = JulianDate_to_YYMMDD(current_year - 1,start_jd)
    else:
        start_jd = 365 + (current_julian_day - 7) #if normal non-leap year
        start_date = JulianDate_to_YYMMDD(current_year - 1,start_jd)

#Get end date if week does not straddle new year
if current_julian_day > 1:
    end_jd = current_julian_day - 1;
    end_date = JulianDate_to_YYMMDD(current_year,end_jd)
#Get end date is week straddles new year
if current_julian_day < 2:
    if calendar.isleap(current_year - 1): 
        end_jd = 366 + (current_julian_day - 1) #if leap year then Julian day count is one more
        end_date = JulianDate_to_YYMMDD(current_year - 1,end_jd)
    else:
        end_jd = 365 + (current_julian_day - 1) #if normal non-leap year
        end_date = JulianDate_to_YYMMDD(current_year - 1,end_jd)


print('Weekly Wx Report for ' + start_date + ' to ' + end_date+'\n')

# Load files for each day in the week and calculate statistics for each day
#
# Creates data table to be populated with appropriate stats for the week. Each
# element represents one day of the week with the stats (9 values) for each day
wx_stats = [[' ' for j in range(9)] for i in range(8)]
wx_stats[0][0] = 'Day\t'
wx_stats[1][0] = 'Monday'
wx_stats[2][0] = 'Tuesday'
wx_stats[3][0] = 'Wednesd'
wx_stats[4][0] = 'Thursday'
wx_stats[5][0] = 'Friday'
wx_stats[6][0] = 'Saturday'
wx_stats[7][0] = 'Sunday'

wx_stats[0][1] = 'Date\t\t'
wx_stats[0][2] = 'Day of Year'
wx_stats[0][3] = 'Wind Hi (m/s)'
wx_stats[0][4] = 'Wind Lo (m/s)'
wx_stats[0][5] = 'Temp Hi (\xb0C)'
wx_stats[0][6] = 'Temp Lo (\xb0C)'
wx_stats[0][7] = 'Low Wind\t'
wx_stats[0][8] = 'North Wind'

low_winds_record = []
low_winds_record.append(['Date and Time\t','Wind Speed'])
north_winds_record = []
north_winds_record.append(['Date and Time\t','Wind Dir'])

all_high_winds = []
all_low_winds = []
all_high_temps = []
all_low_temps = []

all_winds = []
all_temps = []
counter = 1
if start_jd < end_jd: # Check if week straddles new year
    # iterate through week, day by day
    for ii in range(start_jd,end_jd+1):
        wind_speed = []
        wind_dir = []
        temp = []
        time_list = []
        
        date_str = JulianDate_to_YYMMDD(current_year,ii)
        wx_stats[counter][1] = str(date_str)
        wx_stats[counter][2] = str(ii) 
        # Attempt to open the weather file for the day of focus
        try: 
            f = open('//Server/FTP/data/GEOSummit/NOAA/MetData/Sonic Data/NOAAWx_'+date_str+'.dat', 'r')
        except Exception as e:
            # If daily file is missing, skip to the next day
            counter = counter + 1
            print('Error opening file for '+date_str+'. '+str(e))
            continue
        
        header1 = f.readline()
        header2 = f.readline()
        for line in f:
            line = line.strip()
            columns = line.split(',')
            # Parse columns of weather data line
            wind_speed.append(float(columns[1])) 
            wind_dir.append(float(columns[2]))
            temp.append(float(columns[5]))
            time_list.append(str(columns[0]).strip('"'))
        
        wx_stats[counter][3]= str(np.max(wind_speed))
        all_high_winds.append(np.max(wind_speed))
        wx_stats[counter][4]= str(np.min(wind_speed))
        all_low_winds.append(np.min(wind_speed))
        wx_stats[counter][5]= str(np.max(temp))
        all_high_temps.append(np.max(temp))
        wx_stats[counter][6]= str(np.min(temp))
        all_low_temps.append(np.min(temp))
        
        all_winds.append(wind_speed)
        all_temps.append(temp)

        #Check for low winds. Low winds are less than 1 m/s (2 knts)
        t_low_winds = []
        low_winds = []
        low_winds_flag = 'No'
        
        for v in range(0,len(wind_speed)):
            if wind_speed[v] < 1:
                t_low_winds.append(str(time_list[v])+'\t')
                low_winds.append(str(wind_speed[v]))
                low_winds_flag = 'Yes'
                
        wx_stats[counter][7] = low_winds_flag
        if low_winds_flag=='Yes':
            low_winds_record.append([t_low_winds,low_winds])
        
        
        #Check for north winds. North winds are <= 72 degrees or >= 342 degrees
        t_north_winds = []
        north_winds = []
        north_winds_flag = 'No'        

        for d in range(0,len(wind_dir)):
            if wind_dir[d] <= 72 or wind_dir[d] >= 342:
                t_north_winds.append(str(time_list[d])+'\t')
                north_winds.append(str(wind_dir[d]))
                north_winds_flag = 'Yes'
                
        wx_stats[counter][8] = north_winds_flag
        if north_winds_flag=='Yes':
            north_winds_record.append([t_north_winds,north_winds])
                  
        counter = counter + 1
        f.close()

#Calculate weekly stats
#Calcualte weekly wind high
weekly_wind_high = np.max(all_high_winds)
weekly_wind_low = np.min(all_low_winds)
weekly_temp_high = np.max(all_high_temps)
weekly_temp_low = np.min(all_low_temps)

new_all_winds = [i[0] for i in all_winds]
weekly_av_wind = np.mean(new_all_winds)
new_all_temps = [i[0] for i in all_temps]
weekly_av_temp = np.mean(new_all_temps)

#Creates text file weeklyWx_YYYY-MM-DD.TXT
report = open('weekly/weeklyWx_'+end_date+'.txt','w')
report.write('\t'.join(wx_stats[0][0:9])+'\n')
report.write('\t\t'.join(wx_stats[1][0:9])+'\n')
report.write('\t\t'.join(wx_stats[2][0:9])+'\n')
report.write('\t\t'.join(wx_stats[3][0:9])+'\n')
report.write('\t\t'.join(wx_stats[4][0:9])+'\n')
report.write('\t\t'.join(wx_stats[5][0:9])+'\n')
report.write('\t\t'.join(wx_stats[6][0:9])+'\n')
report.write('\t\t'.join(wx_stats[7][0:9])+'\n')

report.write('\nWeekly High Wind (m/s):  %.2f'%weekly_wind_high)
report.write('\nWeekly Low Wind (m/s):  %.2f'%weekly_wind_low)
report.write('\nWeekly Average Wind (m/s):  %.2f'%weekly_av_wind)
report.write('\n\nWeekly High Temp (\xb0C):  %.2f'%weekly_temp_high)
report.write('\nWeekly Low Temp (\xb0C):  %.2f'%weekly_temp_low)
report.write('\nWeekly Average Temp (\xb0C):  %.2f'%weekly_av_temp)


report.write('\n\n'+'Low Winds\n\n')
report.write('\t'.join(low_winds_record[0])+'\n')  
for day in range(1,len(low_winds_record)):
    for tt in range(0,len(low_winds_record[day][0])):
        report.write(low_winds_record[day][0][tt]+'\t'+low_winds_record[day][1][tt]+'\n')
    report.write('\n')

report.write('\n\n'+'North Winds\n\n')
report.write('\t'.join(north_winds_record[0])+'\n')      
for day in range(1,len(north_winds_record)):
    for tt in range(0,len(north_winds_record[day][0])):
        report.write(north_winds_record[day][0][tt]+'\t'+north_winds_record[day][1][tt]+'\n')
    report.write('\n')
    
report.close()
print('\nWeather data processed and report generated.')

